/* ttestpow.do - does power estimation for 2 group t-test

based on poispow.do from http://www.stata.com/support/faqs/statistics/power-by-simulation/

model: mu = b0 + b1*x  where x is a dummy variable
       y ~ Normal(mu,sd)

Specifically, power is estimated for testing the hypothesis b1 = 0, against a
user-specified alternative for a user-specified sample size. Without loss of
generality, we can assume the true value of b0 is 0 and sd is 1 

In the `args' statement below:

        N is number of simulated datasets
        nobs is the number of observations in each of the two groups 
        b1 is the "true" value of b1 (the alternative hypothesis)
*/

args N nobs b1
drop _all
set obs 2
generate x=0 in 1
replace x=1 in 2
expand `nobs'
sort x
generate mu=`b1'*x   
save tempx, replace

/* Note: Here, I generated my "x" and mu-values and stored them in a dataset
-tempx- so that the same values could be used throughout the simulation  */

/* set up a dataset with N observations which will eventually contain N 
"p"-values obtained by testing b1=0.  */
drop _all
set obs `N'
generate pv = .

/* Loop through the simulations */

local i 0
while `i' < `N' {
     local i=`i'+1
     preserve

     use tempx,clear        /* get the n = 2*nobs observations of x 
                               and the mean mu into  memory  */
     gen xn = rnormal(mu,1) /* generate n obs. of a Std Normal(mu)random
                               variable in variable -xn- */
     quietly regress xn x           /* do the regression */
     matrix V=e(V)          /* get the standard-error matrix */
     matrix b=e(b)          /* get the vector of estimated coefficients */
     scalar tv=b[1,1]/sqrt(V[1,1])          /* the "t"-ratio */
     scalar pv = 2*(1-normal(abs(tv)))    /* the p-value  */
     restore /* get back original dataset with N observations */
     quietly replace pv=scalar(pv) in `i'           /* set pv to the p-value for
                                               the ith simulation */
      _dots `i' 0
}

/*The dataset in memory now contains N simulated p-values. To get an
estimate of the power, say for alpha=.05, just calculate the proportion
of pv's that are less than 0.05:  */

count if pv<.05
scalar power=r(N)/`N'
scalar n=2*`nobs'
noisily display "Power is " %8.4f scalar(power) " for a sample size of " /*
	*/ scalar(n) " and alpha = .05"
exit
